@@ -161,39 +161,88 @@ setClass("corrmatrix", representation("matrix", stdDev = "numeric"))
 setClass("cscBlocked", representation(p = "integer", i = "integer", x = "array"))
 
 ## Block/block sparse symmetric matrices
-setClass("bbSparseSy", representation(x = "list", uplo = "character"))
+setClass("bbSparseSy", representation(x = "list", uplo = "character"),
+         validity = function(object) {
+             ul <- object@uplo
+             if (length(ul) != 1 || nchar(ul[1]) != 1 ||
+                 !toupper(ul[1]) %in% c('L', 'U'))
+                 return("uplo slot must be 'U' or 'L'")
+             xl <- object@x
+             nf <- as.integer((-1 + sqrt(1 + 8 * length(xl))) / 2)
+             if (length(xl) != ((nf * (nf + 1))/2))
+                 return("x slot has incorrect length")
+             if (any(unlist(lapply(xl, class)) != "cscBlocked"))
+                 return("x slot must be a list of cscBlocked objects")
+             TRUE
+         })
 
 ## Block/block cross tabulation
-setClass("bbCrosstab", contains = "bbSparseSy")
+#setClass("bbCrosstab", contains = "bbSparseSy")
 
 ## Block/block sparse triangular matrices
 setClass("bbSparseTr", representation(x = "list", uplo = "character",
-                                      diag = "character"))
+                                      diag = "character"),
+         validity = function(object) {
+             ul <- object@uplo
+             if (length(ul) != 1 || nchar(ul[1]) != 1 ||
+                 !toupper(ul[1]) %in% c('L', 'U'))
+                 return("uplo slot must be 'U' or 'L'")
+             dd <- object@diag
+             if (length(dd) != 1 || nchar(dd[1]) != 1 ||
+                 !toupper(dd[1]) %in% c('N', 'U'))
+                 return("diag slot must be 'N' or 'U'")
+             xl <- object@x
+             nf <- as.integer((-1 + sqrt(1 + 8 * length(xl))) / 2)
+             if (length(xl) != ((nf * (nf + 1))/2))
+                 return("x slot has incorrect length")
+             if (any(unlist(lapply(xl, class)) != "cscBlocked"))
+                 return("x slot must be a list of cscBlocked objects")
+             TRUE
+         })
+
 
 ## Block/block L matrix
-setClass("bbLmat", representation(Linv = "list"), contains = "bbSparseTr")
+setClass("bbLmat", representation(Linv = "list"), contains = "bbSparseTr",
+         validity = function(object) {
+             linv <- object@linv
+             xl <- object@x
+             nf <- as.integer((-1 + sqrt(1 + 8 * length(xl))) / 2)
+             if (length(linv) != nf)
+                 return("x and Linv slots have inconsistent length")
+             if (any(unlist(lapply(xl, class)) != "cscBlocked"))
+                 return("x slot must be a list of cscBlocked objects")
+             TRUE
+         })
+             
          
 ## Representation of a linear mixed effects model
-setClass("lmeRep",
+setClass("lmer",
          representation(
-                        D = "list",     # list of diagonal factors (lower triangle)
+                        flist = "list", # list of grouping factors
+                        perm = "list",  # list of permutations of levels (0-based)
+                        Parent = "list",# list of Parent arrays for ZZpO
+                        D = "list",     # list of diagonal factors (upper triangle)
+                        bVar = "list",  # list of conditional variance factors (upper triangle)
                         L = "list",     # list of blocks of L
-                        Linv = "list",  # list of blocks of L^{-1}
+                        ZZpO = "list",  # list of diagonal blocks of Z'Z+Omega
                         Omega = "list", # list of relative precision matrices
+                        REML = "logical", 
                         RXX = "matrix", # Augmented RXX component or its inverse
                         RZX = "matrix", # Augmented RZX component or its inverse
                         XtX = "matrix", # Original X'X matrix
-                        ZZx = "list",   # list of blocks of Z'Z
+                        ZtZ = "list",   # list of blocks of Z'Z
                         ZtX = "matrix", # Original Z'X matrix
                         cnames = "list",# column names of model matrices
                         devComp = "numeric", # Components of deviance
                         deviance = "numeric", # Current deviance (ML and REML)
-                        levels = "list",# names of levels of grouping factors
                         nc = "integer", # number of columns in (augmented)
                                         # model matrices and number of observations
+                        Gp = "integer", # Pointers to groups of rows in RZX
                         status = "logical",
                         call = "call"   # omit this after debugging phase
                         ),
-         validity = function(object)
-         .Call("lmeRep_validate", object))
+         validity = function(object) {
+             .Call("lmer_validate", object)
+         })
+
 
